#include "imagenetwork.h"

ImageNetwork::ImageNetwork()
{
    url_ = QUrl("http://hive.dy.fi/~users/qtcodecamp/process.php");
    manager_ = new QNetworkAccessManager();
    connect(manager_, SIGNAL(finished(QNetworkReply*)),
             this, SLOT(replyFinished(QNetworkReply*)));

    currentState_ = 0;
}
ImageNetwork::~ImageNetwork()
{

}
void ImageNetwork::login(QString username, QString password)
{

    username_ = username;
    password_ = password;

    QNetworkRequest request;
    QUrl params;
    QByteArray data;

    request.setUrl(url_);
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");
    params.addQueryItem("Type", "AUTHENTICATION");
    params.addQueryItem("Username", username_);
    params.addQueryItem("Password", password_);
    data.append(params.toString());
    data.remove(0,1);

    manager_->post(request, data);

    currentState_ = 5;

}
void ImageNetwork::logout()
{
    QNetworkRequest request;
    QUrl params;
    QByteArray data;

    request.setUrl(url_);
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");
    params.addQueryItem("Type","LOGOUT");
    data.append(params.toString());
    data.remove(0,1);

    manager_->post(request, data);
    currentState_ = 5;
}
void ImageNetwork::putImage(QString fullname, QString filename)
{
    QNetworkRequest request;
    QUrl params;
    QByteArray requestData;

    QFile file(fullname);
    if(file.exists() && file.open(QIODevice::ReadOnly))
    {
        QString boundary("---------------------------7d934a10503cc");
        QString crlf(0x0d);
        crlf += 0x0a;

        QString myData;
        myData = "--" + boundary + crlf;
        myData += "Content-Disposition: form-data;";
        myData += "name=\"Image\"; ";
        myData += "filename=\""+filename+"\"";
        myData += crlf + "Content-Type: image/jpeg" + crlf + crlf;

        params.addQueryItem("Type","PUTIMAGE");
        requestData.append(myData);
        requestData.append(file.readAll());
        requestData.append(crlf);
        requestData.append("--" + boundary + crlf);
        requestData.append("Content-Disposition: form-data; name=\"Type\"" + crlf + crlf + "PUTIMAGE" + crlf);
        requestData.append("--" + boundary + crlf);

        file.close();

        request.setUrl(QUrl("http://hive.dy.fi/~users/qtcodecamp/process.php"));
        request.setHeader(QNetworkRequest::ContentTypeHeader, QString("multipart/form-data; boundary="+boundary));
        request.setHeader(QNetworkRequest::ContentLengthHeader, requestData.length());
        manager_->post(request, requestData);
        currentState_ = 3;
   }
}

void ImageNetwork::getImage(QString filename)
{
    QNetworkRequest request;
    QUrl params;
    QByteArray data;

    request.setUrl(url_);
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");

    params.addQueryItem("Type","GETIMAGE");
    params.addQueryItem("Filename",filename);
    data.append(params.toString());
    data.remove(0,1);

    manager_->post(request, data);
    currentState_ = 2;
}

void ImageNetwork::getFileList()
{
    QNetworkRequest request;
    QUrl params;
    QByteArray data;

    request.setUrl(url_);
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");

    params.addQueryItem("Type","GETIMAGELIST");
    data.append(params.toString());
    data.remove(0,1);

    manager_->post(request, data);
    currentState_ = 1;
}

void ImageNetwork::getSimilar()
{
    QNetworkRequest request;
    QUrl params;
    QByteArray data;

    request.setUrl(url_);
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");

    params.addQueryItem("Type","GETSIMILAR");
    data.append(params.toString());
    data.remove(0,1);

    manager_->post(request, data);
    currentState_ = 4;
}

void ImageNetwork::replyFinished(QNetworkReply* reply)
{
    QByteArray data = reply->readAll();
    QString stringData(data);

    QVariant rawContentType = reply->header(QNetworkRequest::ContentTypeHeader);
    QVariant rawContentSize = reply->header(QNetworkRequest::ContentLengthHeader);
    QString fileSize = rawContentSize.toString();
    QString fileType = rawContentType.toString();

    QString replyType;
    int replyValue;

    QTextStream ts( &data );
    ts >> replyType;
    ts >> replyValue;

    if(fileType == "text/html")
    {
        if(replyValue == 1 && replyType == "AUTHENTICATION")
        {
            emit loggedIn();
            isLoggedIn_ = true;
            currentState_ = 0;
        }
        else if(replyValue == 1 && replyType == "PUTIMAGE")
        {
            emit imageUploadReady();

            currentState_ = 0;
        }
        else if(replyValue == 1 && replyType == "GETIMAGELIST")
        {
            QString datas = ts.readAll();
            emit imageListReady(datas);
        }
        else if(replyValue == 1 && replyType == "GETSIMILAR")
        {
            QString datas = ts.readAll();
            datas.remove(0,1);
            emit similarListReady(datas);
        }
        else
        {
            QString str;
            QTextStream ts(&str);
            QMessageBox msgBox;

            ts << "Failed: " << stringData;
            ts << "(" << replyType << "|" << replyValue << ")";
            msgBox.setText(str);
            msgBox.exec();
        }
    }
    else if(fileType == "image/jpg")
    {
        QByteArray cdheader = reply->rawHeader( "Content-Disposition" );
        cdheader.remove(0,22);
        cdheader.remove(cdheader.size()-1,1);

        emit imageReady(data, QString(cdheader));
    }
    else
    {
        emit error("Error: " + stringData);
    }
}
/*
void ImageNetwork::imageReady(QByteArray data)
{

}

void ImageNetwork::imageListReady(QString* data)
{

}

void ImageNetwork::similarListReady(QString* data)
{

}
*/
/*
void ImageNetwork::replyFinished(QNetworkReply* reply)
{
    QByteArray data = reply->readAll();
    QString stringData(data);

    QVariant rawContentType = reply->header(QNetworkRequest::ContentTypeHeader);
    QVariant rawContentSize = reply->header(QNetworkRequest::ContentLengthHeader);

    QString fileSize = rawContentSize.toString();
    QString fileType = rawContentType.toString();

    QString replyType;
    int replyValue;

    QTextStream ts( &data );
    ts >> replyType;
    ts >> replyValue;

    if(fileType == "text/html")
    {
        if(replyValue == 1 && replyType == "AUTHENTICATION")
        {

            state_ = 1; // Logged
            QNetworkRequest request;
            QUrl params;
            QByteArray data;

            QFile file("C:\\temp\\images\\a.jpg");
            if(file.exists() && file.open(QIODevice::ReadOnly))
            {
                QByteArray requestData;

                QString boundary("---------------------------7d934a10503cc");
                QString crlf(0x0d);
                crlf += 0x0a;

                QString myData;
                myData = "--" + boundary + crlf;
                myData += "Content-Disposition: form-data;";
                myData += "name=\"Image\"; ";
                myData += "filename=\"a.jpg\"";
                myData += crlf + "Content-Type: image/jpeg" + crlf + crlf;

                params.addQueryItem("Type","PUTIMAGE");
                requestData.append(myData);
                requestData.append(file.readAll());
                requestData.append(crlf);
                requestData.append("--" + boundary + crlf);
                requestData.append("Content-Disposition: form-data; name=\"Type\"" + crlf + crlf + "PUTIMAGE" + crlf);
                requestData.append("--" + boundary + crlf);

                file.close();

                request.setUrl(QUrl("http://hive.dy.fi/~users/qtcodecamp/process.php"));
                request.setHeader(QNetworkRequest::ContentTypeHeader, QString("multipart/form-data; boundary="+boundary));
                request.setHeader(QNetworkRequest::ContentLengthHeader, requestData.length());
                manager_->post(request, requestData);
           }
        }
        else if(replyValue == 1 && replyType == "PUTIMAGE")
        {

            QNetworkRequest request;
            QUrl params;
            QByteArray data;

            request.setUrl(url_);
            request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");

            params.addQueryItem("Type","GETIMAGE");
            params.addQueryItem("Filename","a.jpg");
            data.append(params.toString());
            data.remove(0,1);

            manager_->post(request, data);

        }
        else if(replyValue == 1 && replyType == "GETIMAGELIST")
        {
            QMessageBox msgBox;
            msgBox.setText("GET: " + stringData);
            msgBox.exec();
        }
        else if(replyValue == 1 && replyType == "GETSIMILAR")
        {
            QString datas = ts.readAll();
            datas.remove(0,1);
            QFile file("C:\\temp\\test.txt");
            file.open(QIODevice::WriteOnly);
            file.write(datas.toAscii(), datas.size());
            file.close();

        }
        else
        {
            QString str;
            QTextStream ts(&str);
            QMessageBox msgBox;

            ts << "Failed: " << stringData;
            ts << "(" << replyType << "|" << replyValue << ")";
            msgBox.setText(str);
            msgBox.exec();
        }
    }
    else if(fileType == "image/jpg")
    {
        QFile image("C:\\temp\\images\\b.jpg");
        image.open(QIODevice::WriteOnly);
        image.write(data.constData(), data.size());
        image.close();


        // This is most likely an image

        QNetworkRequest request;
        QUrl params;
        QByteArray data;

        request.setUrl(url_);
        request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");

        params.addQueryItem("Type","GETSIMILAR");
        data.append(params.toString());
        data.remove(0,1);

        manager_->post(request, data);


        QMessageBox msgBox;
        QString str;
        QTextStream ts(&str);
        ts << "Image ";
        ts << data.size() << " - " << " .";
        ts << fileSize;
        msgBox.setText(str);
        msgBox.exec();

    }
    else
    {
        QMessageBox msgBox;
        msgBox.setText("Failed " + fileType + stringData);
        msgBox.exec();
    }
}
*/
