#ifndef IMAGENETWORK_H
#define IMAGENETWORK_H

#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkRequest>
#include <QtNetwork/QNetworkReply>
#include <QtGui/QMessageBox>
#include <QUrl>
#include <QTextStream>
#include <QVariant>
#include <QFile>
#include <QProgressDialog>
#include <QObject>


class ImageNetwork : public QObject
{
    Q_OBJECT
public:
    ImageNetwork();
    ~ImageNetwork();

    void login(QString username, QString password);
    void logout();

    void putImage(QString fullname, QString filename);
    void getImage(QString filename);

    void getFileList();
    void getSimilar();

    int getState() { return currentState_; };
private:
    QNetworkAccessManager* manager_;
    QString username_, password_;

    QProgressDialog* progress_;

    QUrl url_;

    bool isLoggedIn_; // 0 = no log
    int currentState_; // 0 = idle, 1 = getfilelist, 2 = getimage, 3 = putimage, 4 = getsimilar, 5 = login

signals:
    void loggedIn();
    void imageUploadReady();
    void imageReady(QByteArray,QString filename);
    void imageListReady(QString);
    void similarListReady(QString);
    void error(QString);
public slots:
    void replyFinished(QNetworkReply* reply);

};

#endif // IMAGENETWORK_H
