#include "imageview.h"

ImageView::ImageView(QWidget *parent) :
    QWidget(parent)
{
        parent_ = parent;
        this->currentStackId = 0;
}

int ImageView::showImageFolder(QStringList stackFile, QString imgDir)
{
    QPixmap* newImage = new QPixmap();

    if( newImage->load( imgDir + "/" + stackFile.at(0) ) )
    {
        *newImage = newImage->scaled(400, 400,Qt::KeepAspectRatio);
        pixmap_ = newImage;
        return 0;
    }
    return -1;
}

void ImageView::createImageView( Stack* imgStack, QString imgDir)
{
    /* Set current image initially to zero */
    this->currentImageId = 0;
    this->imgStack_ = imgStack;
    this->imgDir_ = imgDir;

    /* Create new grid layout for our fancy custom stack image widgets  */
    QGridLayout* grid_ = new QGridLayout(parent_);

    imageLabel_ = new QLabel();
    pixmap_ = new QPixmap(imgDir + "/" + imgStack->getImage( this->currentImageId ) );
    imageLabel_->setBackgroundRole(QPalette::Base);
    imageLabel_->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
    pixmap_->scaled(imageLabel_->size(),Qt::KeepAspectRatio);
    imageLabel_->setScaledContents(true);
    imageLabel_->setPixmap( *pixmap_ );


    QPushButton* bLeft = new QPushButton("<", this->parent_);
    bLeft->setFixedHeight(100);
    grid_->addWidget(bLeft,0,0,1,1,Qt::AlignCenter);
    connect(bLeft, SIGNAL(clicked()), this, SLOT(bLeftClicked()) );

    grid_->addWidget(imageLabel_,0,1,2,1,Qt::AlignCenter);

    QPushButton* bRight = new QPushButton(">", this->parent_);
    bRight->setFixedHeight(100);
    grid_->addWidget(bRight,0,2,1,1,Qt::AlignCenter);
    connect(bRight, SIGNAL(clicked()), this, SLOT(bRightClicked()));

    QPushButton* bHome = new QPushButton("HM", this->parent_);
    grid_->addWidget(bHome,1,0,1,1,Qt::AlignCenter);
    connect(bHome, SIGNAL(clicked()), this, SLOT(bHomeClicked()) );
/*
    QPushButton* bInfo = new QPushButton("i", this->parent_);
    grid_->addWidget(bInfo,1,2,1,1,Qt::AlignCenter);
*/
    grid_->setColumnMinimumWidth(0,5);
    grid_->setColumnMinimumWidth(2,5);
    grid_->setRowMinimumHeight(1,5);
    grid_->setColumnStretch(1,400);
    grid_->setRowStretch(0,400);

    this->setLayout(grid_);

}

void ImageView::die()
{
    this->destroy();
}

/*
 * SIGNALS / EVENTS
 */

void ImageView::bHomeClicked()
{
    emit exited();
}

void ImageView::bInfoClicked(){}

void ImageView::bLeftClicked()
{
    if(this->currentImageId > 0)
    {
        (this->currentImageId)--;
        pixmap_->load( this->imgDir_ + "/" + this->imgStack_->getImage(this->currentImageId) );
        imageLabel_->setBackgroundRole(QPalette::Base);
        //imageLabel_->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
        imageLabel_->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
        imageLabel_->setScaledContents(true);
        imageLabel_->setPixmap( *pixmap_ );
    }
}

void ImageView::bRightClicked()
{
    if( this->currentImageId < this->imgStack_->getNImages()-1 )
    {
        (this->currentImageId)++;
        pixmap_->load( this->imgDir_ + "/" + this->imgStack_->getImage(this->currentImageId) );
        imageLabel_->setBackgroundRole(QPalette::Base);
        //imageLabel_->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
        imageLabel_->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
        imageLabel_->setScaledContents(true);
        imageLabel_->setPixmap( *pixmap_ );
    }
}
