#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    SettingsView* settingsView = new SettingsView();
    settingsView->createSettingsView();
    connect(settingsView, SIGNAL(submitted()), this, SLOT(processSettings()));

    this->setCentralWidget( settingsView );




    in_ = new ImageNetwork();
    connect(in_, SIGNAL(loggedIn()), this, SLOT(processLogin()));
    connect(in_, SIGNAL(similarListReady(QString)), this, SLOT(processSimilarList(QString)));
    connect(in_, SIGNAL(imageListReady(QString)), this, SLOT(processFileList(QString)));
    connect(in_, SIGNAL(imageUploadReady()), this, SLOT(processImageUpload()));
    connect(in_, SIGNAL(imageReady(QByteArray,QString)), this, SLOT(processImageDownload(QByteArray,QString)));
}

MainWindow::~MainWindow()
{
}

int MainWindow::initializeImageCollection(QWidget* parent, QString imgDir)
{
    /* Get a directory listing for the directory of full sized images */
    QDir* dirFull = new QDir(imgDir+"full/","*.jpg");
    QDir* dirThumbs = new QDir(imgDir+"medium/","*.jpg");
    QStringList imgFullList = dirFull->entryList();
    QStringList imgThumbsList = dirThumbs->entryList();
    QImage newI;
    QImage outI;

    /* If there is not any thumbnails, we have to generate them all */
    if( imgThumbsList.size() == 0 )
    {
        for(int i=0; i < imgFullList.size(); i++)
        {
            newI.load( imgDir + "full/" + imgFullList.at(i) );
            outI = newI.scaled( QSize(600,600), Qt::KeepAspectRatio);
            outI.save( imgDir + "medium/" + imgFullList.at(i) );
        }
    }

    for(int i=0, j=0; i < imgFullList.size(); ++i )
    {
/* This does not update so it is pretty useless
   TODO: Loading bar would be nice!
        myBox->setText( imgDir + imageList.at(i) );
        myBox->update();
        grid->addWidget(myBox,0,0,Qt::AlignCenter);
        grid->update();
*/
        /* Find out if there is already a thumbnail */
        if( QString::compare(imgFullList.at(i), imgThumbsList.at(j)) != 0 )
        {
            newI.load( imgDir + "full/" + imgFullList.at(i) );
            outI = newI.scaled( QSize(600,600), Qt::KeepAspectRatio);
            outI.save( imgDir + "medium/" + imgFullList.at(i) );
        }
        else
            j++;
    }
    return 0;
}

/*
 * Genenerate stacks from input file
 */
int MainWindow::generateStacks(QString stackFile)
{
    /* newStack is a image stack */
//    Stack newStack;
    int cs=0;

    QFile* fp = new QFile(stackFile);
    if( !fp->open(QIODevice::ReadOnly | QIODevice::Text) )
        return -1;

    /* Open stack file as input text stream */
    QTextStream in(fp);

    /* Read the bloody stack file */
    while( !in.atEnd() )
    {
        QString line = fp->readLine();
        /* Create new stack using all filenames on the row */
        QStringList imgList = line.split(" ");

        if( imgList.count() > 0 )
        {
            /* Create a new stack */
            Stack newStack = Stack( imgList, cs++ );
            this->stackList_.append( newStack );
        }
    }
    return 0;
}

int MainWindow::buildMainViewFromFile(QWidget *parent, QGridLayout *grid, QString stackFile)
{
// UNUSED    QMessageBox* myBox = new QMessageBox( parent );
    QString imgDir = "/home/tekinnun/imcat/images/teemu/medium/";
    ImageStack* newStack;
    int x=0, y=0;
    QFile* fp = new QFile(stackFile);
    if( !fp->open(QIODevice::ReadOnly | QIODevice::Text) )
        return -1;

    QTextStream in(fp);
    /* Read the bloody stack file */
    while( !in.atEnd() )
    {
        QString line = fp->readLine();
        /* Create new stack using all filenames on the row */
        QStringList imgList = line.split(" ");
        if( imgList.count() > 0 )
        {
            /* Create new widget */
            newStack = new ImageStack( parent );
            /* Add some images to the widget */
            for(int i=0; i<=3 && i<imgList.count(); i++)
            {
                newStack->addImage( imgDir + imgList.at(i) );
/*
                myBox->setText( imgDir + imgList.at(i) );
                grid->addWidget(myBox,0,0,Qt::AlignCenter);
*/
            }
            /* Finally add new widget to the grid */
            grid->addWidget(newStack,y,x,Qt::AlignCenter);
            x++;
            if(x==3)
            {
                x=0;
                y++;
            }
        }
    }

    return 0;
}

void MainWindow::imageView(int view)
{
        imgView_ = new ImageView();
        Stack* tmpStackP = (Stack*) &( this->stackList_.at(view) );
        imgView_->createImageView( tmpStackP, realImageDir_);
        connect(imgView_, SIGNAL(exited()), this, SLOT(mainView()));

        this->setCentralWidget(imgView_);
        this->stackView_->die();
}
void MainWindow::mainView()
{
        scrollArea_ = new QScrollArea;
        scrollArea_->setBackgroundRole(QPalette::Dark);
        stackView_ = new StackView();
        stackView_->createStackView( this->stackList_, realImageDir_);
        connect(stackView_, SIGNAL(stackClicked(int)), this, SLOT(imageView(int)));

        scrollArea_->setWidget(stackView_);
        this->setCentralWidget(scrollArea_);
        this->imgView_->die();
}
void MainWindow::processImageUpload()
{
    if(progressCurrent_ < progressTotal_ )
    {
        progressCurrent_++;
        progressBar_->setValue(progressCurrent_);
    }
    else
    {
        in_->getSimilar();
    }
}

void MainWindow::processSettings()
{
    QSettings* userSettings = new QSettings("Apinaorkesteri", "iMagine");
    username_ = ((userSettings->value("UserSettings/Username")).toString());
    password_ = ((userSettings->value("UserSettings/Password")).toString());
    imageDir_ = ((userSettings->value("UserSettings/ImageDir")).toString());
    stackFile_ = imageDir_+"/myStack.xxx";
    realImageDir_ = imageDir_+"/"+username_+"/medium";
    in_->login(username_,password_);
}

void MainWindow::processLogin()
{
    QDir dir(realImageDir_);
    dir.mkpath(realImageDir_);

    in_->getFileList();

}

void MainWindow::processFileList(QString data)
{
    QTextStream ts(&data);
    QStringList serverFiles;
    QStringList clientFiles;
    while( !ts.atEnd() )
    {
        QString str;
        ts >> str;
        serverFiles.push_back(str);
    }
    QDir* dir = new QDir(realImageDir_,"*.jpg");
    clientFiles = dir->entryList();
    progressTotal_ = 0;
    progressCurrent_ = 0;

    //QProgressDialog progress("Syncing files...","Cancel", 0, 10,this);

    progressBar_ = new QProgressBar;
    progressBar_->setMinimum(0);
    progressBar_->setMaximum(10);
    progressBar_->setValue(1);
    progressBar_->setFixedHeight(20);

    this->setCentralWidget(progressBar_);

    int srvFiles = serverFiles.size();
    for(int i=0; i < clientFiles.size(); ++i )
    {
        bool found = false;
        for(int j=0; j < srvFiles; ++j)
        {
            if(QString::compare(clientFiles.at(i), serverFiles.at(j)) == 0)
            {
                found = true;
            }
        }
        if(!found)
        {
            // Upload to server
            progressTotal_ += 1;
            in_->putImage(realImageDir_ + "/" +clientFiles.at(i),clientFiles.at(i));
            serverFiles.push_back(clientFiles.at(i));
        }
    }
    for(int i=0; i < serverFiles.size(); i++ )
    {
        bool found = false;

        for(int j=0; j < clientFiles.size(); j++)
        {
            if(QString::compare(serverFiles.at(i), clientFiles.at(j)) == 0)
            {
                found = true;
                break;
            }
        }
        if(!found)
        {
            // download from server
            progressTotal_ += 1;
            in_->getImage(serverFiles.at(i));
        }
    }
    progressBar_->setMaximum(progressTotal_);
    progressBar_->setValue(progressCurrent_);
    processImageUpload();
}
void MainWindow::processImageDownload(QByteArray data, QString filename)
{
    QFile file(realImageDir_+"/"+filename);
    file.open(QIODevice::WriteOnly);
    file.write(data, data.size());
    file.close();
    if(progressCurrent_ < progressTotal_ )
    {
        progressCurrent_++;
        progressBar_->setValue(progressCurrent_);
    }
    else
    {
        in_->getSimilar();
    }
}

void MainWindow::processSimilarList(QString data)
{
    QFile file(stackFile_);
    file.open(QIODevice::WriteOnly);
    file.write(data.toAscii(), data.size());
    file.close();
    /* MAIN VIEW CODE */

    scrollArea_ = new QScrollArea;
    scrollArea_->setBackgroundRole(QPalette::Dark);

    stackView_ = new StackView(this);
    connect(stackView_, SIGNAL(stackClicked(int)), this, SLOT(imageView(int)));

    imgView_ = new ImageView();
    generateStacks(stackFile_);
    stackView_->createStackView( this->stackList_, realImageDir_);

    scrollArea_->setWidget(stackView_);

    this->setCentralWidget(scrollArea_);

}
