#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QLabel>
#include <QImage>
#include <QPixmap>
#include <QPushButton>
#include <QGridLayout>
#include <QScrollArea>
#include <QString>
#include <QDir>
#include <QMessageBox>
#include <QProgressBar>
#include "imagestack.h"
#include "imagenetwork.h"
#include "stackview.h"
#include "stack.h"
#include "imageview.h"
#include "settingsview.h"

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

    int initializeImageCollection(QWidget* parent, QString imgDir);
    int buildMainViewFromFile(QWidget* parent, QGridLayout* grid, QString stackFile);
    int generateStacks(QString stackFile);

public slots:

    // views
    void imageView(int view);
    void mainView();
    void processSettings();

    // network
    void processLogin();
    void processFileList(QString);
    void processSimilarList(QString);
    void processImageUpload();
    void processImageDownload(QByteArray data, QString filename);

private:
    int view_;

    int progressTotal_;
    int progressCurrent_;
    ImageView* imgView_;
    StackView* stackView_;
    QList< Stack > stackList_;
    QScrollArea* scrollArea_;
    ImageNetwork* in_;
    QDir directory_;
    QString username_;
    QString password_;
    QString imageDir_;
    QString realImageDir_;
    QString stackFile_;
    QProgressBar* progressBar_;
};

#endif // MAINWINDOW_H
