#include "stackview.h"

StackView::StackView(QWidget *parent) :
    QWidget(parent)
{
    nColumns_ = 5;
    parent_ = parent;
    stackList_.clear();
}

int StackView::AddStack(QString stackFile, QString imgDir)
{
    /* newStack is a image stack */
    ImageStack* newStack;
    //scrollArea_ = new QScrollArea(parent_);

    /* Create new grid layout for our fancy custom stack image widgets  */
    QGridLayout* grid_ = new QGridLayout;

    QFile* fp = new QFile(stackFile);
    if( !fp->open(QIODevice::ReadOnly | QIODevice::Text) )
        return -1;

    /* Open stack file as input text stream */
    QTextStream in(fp);

    /* Current location on the grid */
    int x=0, y=0;

    /* Read the bloody stack file */
    while( !in.atEnd() )
    {
        QString line = fp->readLine();
        /* Create new stack using all filenames on the row */
        QStringList imgList = line.split(" ");

        /* Add list of the images to the stacklist */
        stackList_.append(imgList);

        if( imgList.count() > 0 )
        {
            /* Create new widget */
            newStack = new ImageStack( parent_ );
            newStack->setId(1);
            /*
             * This connects imagestack to the stackview
             */
            connect(newStack, SIGNAL(clicked(int)), this, SLOT(stackSelected(int)) );


            /* Add some images to the widget */
//            for(int i=0; i<=3 && i<imgList.count(); i++)
            for(int i=0; i<imgList.count(); i++)
            {
                newStack->addImage( imgDir + imgList.at(i) );
            }
            /* Finally add new widget to the grid */
            grid_->addWidget(newStack,y,x,Qt::AlignCenter);
            x++;
            if(x==nColumns_)
            {
                x=0;
                y++;
            }
        }
    }

    setLayout(grid_);
    //scrollArea_->setWidget(this);

    return 0;
}
void StackView::createStackView(QList<Stack> stackList, QString imgDir)
{
    /* We are going to use scroll area so we can generate a HUGE grid layout */


    /* ImageStack Widget */
    ImageStack* newStack;

    /* Create new grid layout for our fancy custom stack image widgets  */
    QGridLayout* grid_ = new QGridLayout;

    /* Current location on the grid */
    int x=0, y=0;

    /* Read the bloody stack file */
    for(QList< Stack >::iterator it=stackList.begin(); it != stackList.end(); ++it)
    {
        /* Create new widget */
        newStack = new ImageStack();
        newStack->setId( (*it).getId() );

        /*
         * This connects imagestack to the stackview
         */
        connect(newStack, SIGNAL(clicked(int)), this, SLOT(stackSelected(int)) );


        /* Add some images to the widget */
        QStringList imgList = (*it).getImageList();
        for(int i=0; i < imgList.length() && i <= 3; i++)
        {
            newStack->addImage( imgDir + "/" + (*it).getImage(i) );
        }
        /* Finally add new widget to the grid */
        grid_->addWidget(newStack,y,x,Qt::AlignCenter);
        x++;
        if(x==nColumns_)
        {
            x=0;
            y++;
        }
    }
    this->setLayout(grid_);
    //this->setMinimumSize(1000,1000);
    //scrollArea_->setMaximumSize(100,100);



}

void StackView::setNColumns(int c)
{
    this->nColumns_ = c;
}


QStringList StackView::getImageStackList(int id)
{
    return QStringList("");
}

void StackView::die()
{
this->destroy();
}

/*
 * SIGNALS
 */
void StackView::stackSelected(int value)
{
    emit stackClicked(value);
}

void StackView::resizeEvent(QResizeEvent *event)
{
    if(this->nColumns_ == 5)
        this->nColumns_ = 3;
    else
        this->nColumns_ = 5;

}
